@@ -19,12 +19,11 @@
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, see <http://www.gnu.org/licenses/>.
 
-# Python 2.3 string methods: 'rfind', 'rindex', 'rjust', 'rstrip'
-
 from __future__ import absolute_import, print_function
 
 import errno
 import fcntl
+import importlib.metadata
 import itertools
 import logging
 import mimetypes
@@ -34,40 +33,25 @@ import re
 import shutil
 import signal
 import stat
-import string
 import struct
+import subprocess
 import sys
 import tempfile
 import termios
 import textwrap
 import traceback
+import urllib.parse as urlparse
 from functools import cmp_to_key, total_ordering
 
-# Python 3 compatibility hacks commence
-try:
-    import urlparse
-except ImportError:
-    import urllib.parse as urlparse
-try:
-    import subprocess32 as subprocess
-except ImportError:
-    import subprocess
 
-if sys.version_info[0] >= 3:
-    get_input = input
+def cmp(a, b):
+    return (a > b) - (a < b)
 
-    def cmp(a, b):
-        return (a > b) - (a < b)
-
-else:
-    get_input = raw_input  # noqa: F821
 
 try:
-    set
-except NameError:
-    from sets import Set as set
-
-VERSION = "8.5.3"
+    VERSION = importlib.metadata.version("dtrx")
+except importlib.metadata.PackageNotFoundError:
+    VERSION = "DEVELOPMENT"
 VERSION_BANNER = """dtrx version %s
 Copyright © 2006-2011 Brett Smith <brettcsmith@brettcsmith.org>
 Copyright © 2008 Peter Kelemen <Peter.Kelemen@gmail.com>
@@ -83,11 +67,6 @@ WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 Public License for more details.""" % (VERSION,)
 
-# Python3.6 optparse has a hard time parsing this, so ascii transform it
-if sys.version_info[:2] == (3, 6):
-    VERSION_BANNER = VERSION_BANNER.encode("ascii", errors="ignore").decode("ascii")
-
-
 MATCHING_DIRECTORY = 1
 ONE_ENTRY_KNOWN = 2
 BOMB = 3
@@ -157,7 +136,10 @@ class DirectoryChecker(FilenameChecker):
     free_close = None
 
     def create(self):
-        return tempfile.mkdtemp(prefix=self.original_name + ".", dir=".")
+        dirname = tempfile.mkdtemp(prefix=self.original_name + ".", dir=".")
+        # We want to directory to be relative to current directory
+        dirname = os.path.join(".", os.path.relpath(dirname))
+        return dirname
 
 
 class NonblockingRead(object):
@@ -170,27 +152,9 @@ class NonblockingRead(object):
         flags = fcntl.fcntl(fd, fcntl.F_GETFL)
         fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
 
-    def python_2_readlines(self):
-        # XXX: There seems to be a bug in Python 2 where readline() returns
-        # "IOError: [Errno 11] Resource temporarily unavailable" on a
-        # non-blocking read from a pipe where the output lacks a newline.
-        # It doesn't happen in Python 3, so this hack can be deleted once we
-        # no longer care about Python 2.
-        out = ""
-        try:
-            while True:
-                # read a single byte at a time until we hit IOError
-                out += self.iostream.read(1).decode("ascii", "ignore")
-        except IOError:
-            pass
-        return out.splitlines(True)
-
     def readlines(self):
-        if sys.version_info[0] >= 3:
-            out = self.iostream.readlines()
-            return [line.decode("ascii", "ignore") for line in out]
-        else:
-            return self.python_2_readlines()
+        out = self.iostream.readlines()
+        return [line.decode("ascii", "ignore") for line in out]
 
 
 class ExtractorError(Exception):
@@ -222,9 +186,7 @@ class BaseExtractor(object):
         # option based on what's supported, since this behavior changed
         if encoding in ("lrzip", "lrz"):
             # need to check if this version of lrzip supports the -Q option
-            output = subprocess.check_output(
-                "lrzip --help", stderr=subprocess.STDOUT, shell=True
-            )
+            output = subprocess.check_output("lrzip --help", stderr=subprocess.STDOUT, shell=True)
             if b"-Q" in output:
                 decoder = ["lrzcat", "-Q"]
             else:
@@ -237,6 +199,7 @@ class BaseExtractor(object):
         self.filename = os.path.realpath(filename)
         self.encoding = encoding
         self.ignore_pw = False
+        self.password = None
         self.file_count = 0
         self.included_archives = []
         self.target = None
@@ -262,9 +225,7 @@ class BaseExtractor(object):
         try:
             logger.debug("running command: {}".format(command))
             processes.append(
-                subprocess.Popen(
-                    command, stdin=stdin, stdout=stdout, stderr=subprocess.PIPE
-                )
+                subprocess.Popen(command, stdin=stdin, stdout=stdout, stderr=subprocess.PIPE)
             )
         except OSError as error:
             if error.errno == errno.ENOENT:
@@ -279,10 +240,9 @@ class BaseExtractor(object):
             try:
                 return pipe.wait(timeout=1)
             except subprocess.TimeoutExpired:
-                logging.debug("timeout hit..")
+                logging.debug("timeout hit...")
                 self.timeout_check(pipe)
-                # Verify that we're not trying to extract password-protected
-                # archives in non-interactive mode
+                # Verify that we're not waiting for a password in non-interactive mode
                 if self.pw_prompted and self.ignore_pw:
                     pipe.kill()
                     # Whatever extractor we're using probably left the
@@ -292,16 +252,14 @@ class BaseExtractor(object):
                     self.stderr = ""
                     raise ExtractorError(
                         "cannot extract encrypted archive '%s' in non-interactive mode"
-                        % (self.filename)
+                        " without a password" % (self.filename)
                     )
 
     def send_stdout_to_dev_null(self):
         return True
 
     def run_pipes(self, final_stdout=None):
-        has_output_target = (
-            True if final_stdout or self.send_stdout_to_dev_null() else False
-        )
+        has_output_target = True if final_stdout or self.send_stdout_to_dev_null() else False
         if not self.pipes:
             return
         elif final_stdout is None:
@@ -342,9 +300,9 @@ class BaseExtractor(object):
             self.file_count += len(filenames)
             path = path[start_index:]
             for filename in filenames:
-                if ExtractorBuilder.try_by_mimetype(
+                if ExtractorBuilder.try_by_mimetype(filename) or ExtractorBuilder.try_by_extension(
                     filename
-                ) or ExtractorBuilder.try_by_extension(filename):
+                ):
                     self.included_archives.append(os.path.join(path, filename))
 
     def check_contents(self):
@@ -400,12 +358,8 @@ class BaseExtractor(object):
 
     def check_success(self, got_files):
         error_index, error_code = self.first_bad_exit_code()
-        logger.debug(
-            "success results: %s %s %s" % (got_files, error_index, self.exit_codes)
-        )
-        if self.is_fatal_error(error_code) or (
-            (not got_files) and (error_code is not None)
-        ):
+        logger.debug("success results: %s %s %s" % (got_files, error_index, self.exit_codes))
+        if self.is_fatal_error(error_code) or ((not got_files) and (error_code is not None)):
             command = " ".join(self.pipes[error_index][0])
             self.pw_prompt = False  # Don't silently fail with wrong password
             raise ExtractorError(
@@ -417,10 +371,15 @@ class BaseExtractor(object):
         self.pipe(self.extract_pipe)
         self.run_pipes()
 
-    def extract(self, ignore_passwd=False):
+    def extract(self, ignore_passwd=False, password=None):
         self.ignore_pw = ignore_passwd
+        self.password = password
         try:
-            self.target = tempfile.mkdtemp(prefix=".dtrx-", dir=".")
+            dirname = tempfile.mkdtemp(prefix=".dtrx-", dir=".")
+            # We want to directory to be relative to current directory
+            dirname = os.path.join(".", os.path.relpath(dirname))
+            self.target = dirname
+
         except (OSError, IOError) as error:
             raise ExtractorError("cannot extract here: %s" % (error.strerror,))
         old_path = os.path.realpath(os.curdir)
@@ -478,14 +437,13 @@ class CompressionExtractor(BaseExtractor):
         # compression extensions, even if those files shouldn't actually be
         # handled this way.  So, we call out to the file command to do a quick
         # check and make sure this actually looks like a compressed file.
-        if "compress" not in [
-            match[0] for match in ExtractorBuilder.try_by_magic(self.filename)
-        ]:
+        if "compress" not in [match[0] for match in ExtractorBuilder.try_by_magic(self.filename)]:
             raise ExtractorError("doesn't look like a compressed file")
         yield self.basename()
 
-    def extract(self, ignore_passwd=False):
+    def extract(self, ignore_passwd=False, password=None):
         self.ignore_pw = ignore_passwd
+        self.password = password
         self.content_type = ONE_ENTRY_KNOWN
         self.content_name = self.basename()
         self.contents = None
@@ -564,9 +522,7 @@ class DebExtractor(TarExtractor):
         encoding = mimetypes.guess_type(data_filename)[1]
         if not encoding:
             raise ExtractorError("data.tar file has unrecognized encoding")
-        self.pipe(
-            ["ar", "p", self.filename, data_filename], "extracting data.tar from .deb"
-        )
+        self.pipe(["ar", "p", self.filename, data_filename], "extracting data.tar from .deb")
         self.pipe(self.decoders[encoding], "decoding data.tar")
 
     def basename(self):
@@ -585,9 +541,7 @@ class DebExtractor(TarExtractor):
 
 class DebMetadataExtractor(DebExtractor):
     def prepare(self):
-        self.pipe(
-            ["ar", "p", self.filename, "control.tar.gz"], "control.tar.gz extraction"
-        )
+        self.pipe(["ar", "p", self.filename, "control.tar.gz"], "control.tar.gz extraction")
         self.pipe(["zcat"], "control.tar.gz decompression")
 
 
@@ -637,9 +591,7 @@ class NoPipeExtractor(BaseExtractor):
         extract_fmt_args = {
             "OUTPUT_FILE": os.path.splitext(os.path.basename(self.filename))[0],
         }
-        formatted_extract_commands = [
-            x.format(**extract_fmt_args) for x in self.extract_command
-        ]
+        formatted_extract_commands = [x.format(**extract_fmt_args) for x in self.extract_command]
 
         self.extract_pipe = formatted_extract_commands + [self.filename]
         BaseExtractor.extract_archive(self)
@@ -651,9 +603,18 @@ class NoPipeExtractor(BaseExtractor):
 
 class ZipExtractor(NoPipeExtractor):
     file_type = "Zip file"
-    extract_command = ["unzip", "-q"]
     list_command = ["zipinfo", "-1"]
 
+    @property
+    def extract_command(self):
+        """
+        Returns the extraction command and adds a password if given.
+        """
+        cmd = ["unzip", "-q"]
+        if self.password:
+            cmd.append("-P %s" % (self.password,))
+        return cmd
+
     def is_fatal_error(self, status):
         return (status or 0) > 1
 
@@ -702,20 +663,25 @@ class LZHExtractor(ZipExtractor):
 
 class SevenExtractor(NoPipeExtractor):
     file_type = "7z file"
-    extract_command = ["7z", "x"]
-    list_command = ["7z", "l"]
+    list_command = ["7z", "l", "-ba"]
     border_re = re.compile("^[- ]+$")
+    space_re = re.compile(" ")
+
+    @property
+    def extract_command(self):
+        """
+        Returns the extraction command and adds a password if given.
+        """
+        cmd = ["7z", "x"]
+        if self.password:
+            cmd.append("-p%s" % (self.password,))
+        return cmd
 
     def get_filenames(self):
-        fn_index = None
         for line in NoPipeExtractor.get_filenames(self):
-            if self.border_re.match(line):
-                if fn_index is not None:
-                    break
-                else:
-                    fn_index = string.rindex(line, " ") + 1
-            elif fn_index is not None:
-                yield line[fn_index:]
+            if " " in line:
+                pos = line.rindex(" ") + 1
+                yield line[pos:]
         self.archive.close()
 
     def send_stdout_to_dev_null(self):
@@ -747,7 +713,7 @@ class ZstandardExtractor(NoPipeExtractor):
                 if fn_index is not None:
                     break
                 else:
-                    fn_index = string.rindex(line, " ") + 1
+                    fn_index = line.rindex(" ") + 1
             elif fn_index is not None:
                 yield line[fn_index:]
         self.archive.close()
@@ -809,10 +775,19 @@ class ShieldExtractor(NoPipeExtractor):
 
 class RarExtractor(NoPipeExtractor):
     file_type = "RAR archive"
-    extract_command = ["unrar", "x"]
     list_command = ["unrar", "v"]
     border_re = re.compile("^-+$")
 
+    @property
+    def extract_command(self):
+        """
+        Returns the extraction command and adds a password if given.
+        """
+        cmd = ["unrar", "x"]
+        if self.password:
+            cmd.append("-p%s" % (self.password,))
+        return cmd
+
     def get_filenames(self):
         inside = False
         isfile = True
@@ -843,9 +818,18 @@ class RarExtractor(NoPipeExtractor):
 
 class UnarchiverExtractor(NoPipeExtractor):
     file_type = "RAR archive"
-    extract_command = ["unar", "-D"]
     list_command = ["lsar"]
 
+    @property
+    def extract_command(self):
+        """
+        Returns the extraction command and adds a password if given.
+        """
+        cmd = ["unar", "-D"]
+        if self.password:
+            cmd.append("-p %s" % (self.password,))
+        return cmd
+
     def get_filenames(self):
         output = NoPipeExtractor.get_filenames(self)
         next(output)
@@ -856,10 +840,19 @@ class UnarchiverExtractor(NoPipeExtractor):
 
 class ArjExtractor(NoPipeExtractor):
     file_type = "ARJ archive"
-    extract_command = ["arj", "x", "-y"]
     list_command = ["arj", "v"]
     prefix_re = re.compile(r"^\d+\)\s+")
 
+    @property
+    def extract_command(self):
+        """
+        Returns the extraction command and adds a password if given.
+        """
+        cmd = ["arj", "x", "-y"]
+        if self.password:
+            cmd.append("-g%s" % (self.password,))
+        return cmd
+
     def get_filenames(self):
         for line in NoPipeExtractor.get_filenames(self):
             match = self.prefix_re.match(line)
@@ -876,19 +869,17 @@ class BaseHandler(object):
 
     def handle(self):
         command = "find"
-        status = subprocess.call(
-            [
-                "find",
-                self.extractor.target,
-                "-type",
-                "d",
-                "-exec",
-                "chmod",
-                "u+rwx",
-                "{}",
-                ";",
-            ]
-        )
+        status = subprocess.call([
+            "find",
+            self.extractor.target,
+            "-type",
+            "d",
+            "-exec",
+            "chmod",
+            "u+rwx",
+            "{}",
+            ";",
+        ])
         if status == 0:
             command = "chmod"
             status = subprocess.call(["chmod", "-R", "u+rwX", self.extractor.target])
@@ -899,9 +890,7 @@ class BaseHandler(object):
     def set_target(self, target, checker):
         self.target = checker(target).check()
         if self.target != target:
-            logger.warning(
-                "extracting %s to %s" % (self.extractor.filename, self.target)
-            )
+            logger.warning("extracting %s to %s" % (self.extractor.filename, self.target))
 
 
 # The "where to extract" table, with options and archive types.
@@ -932,9 +921,7 @@ class FlatHandler(BaseHandler):
             if not os.path.isdir(newdir):
                 os.makedirs(newdir)
             for filename in filenames:
-                os.rename(
-                    os.path.join(curdir, filename), os.path.join(newdir, filename)
-                )
+                os.rename(os.path.join(curdir, filename), os.path.join(newdir, filename))
             os.rmdir(curdir)
 
 
@@ -960,9 +947,7 @@ class MatchHandler(BaseHandler):
         )
 
     def organize(self):
-        source = os.path.join(
-            self.extractor.target, os.listdir(self.extractor.target)[0]
-        )
+        source = os.path.join(self.extractor.target, os.listdir(self.extractor.target)[0])
         if os.path.isdir(source):
             checker = DirectoryChecker
         else:
@@ -1008,9 +993,7 @@ class BombHandler(BaseHandler):
 @total_ordering
 class BasePolicy(object):
     try:
-        size = fcntl.ioctl(
-            sys.stdout.fileno(), termios.TIOCGWINSZ, struct.pack("HHHH", 0, 0, 0, 0)
-        )
+        size = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, struct.pack("HHHH", 0, 0, 0, 0))
         width = struct.unpack("HHHH", size)[1]
     except IOError:
         width = 80
@@ -1036,7 +1019,7 @@ class BasePolicy(object):
         while True:
             print("\n".join(question))
             try:
-                answer = get_input(self.prompt)
+                answer = input(self.prompt)
             except EOFError:
                 return self.answers[""]
             try:
@@ -1104,9 +1087,7 @@ class OneEntryPolicy(BasePolicy):
         question.append(" Expected: " + extractor.basename())
         question.append("   Actual: " + extractor.content_name)
         choice_vars = (extractor.content_type, extractor.basename())
-        self.choices = [
-            text % choice_vars[: text.count("%s")] for text in self.choice_template
-        ]
+        self.choices = [text % choice_vars[: text.count("%s")] for text in self.choice_template]
         self.current_policy = self.permanent_policy or self.ask_question(question)
 
     def ok_for_match(self):
@@ -1140,9 +1121,7 @@ class RecursionPolicy(BasePolicy):
 
     def prep(self, current_filename, target, extractor):
         archive_count = len(extractor.included_archives)
-        if (self.permanent_policy is not None) or (
-            (archive_count * 10) <= extractor.file_count
-        ):
+        if (self.permanent_policy is not None) or ((archive_count * 10) <= extractor.file_count):
             self.current_policy = self.permanent_policy or RECURSE_NOT_NOW
             return
         question = self.wrap(
@@ -1162,12 +1141,10 @@ class RecursionPolicy(BasePolicy):
                 break
             print(
                 "\n%s\n"
-                % "\n".join(
-                    [
-                        os.path.join(target, included_root, filename)
-                        for filename in extractor.included_archives
-                    ]
-                )
+                % "\n".join([
+                    os.path.join(target, included_root, filename)
+                    for filename in extractor.included_archives
+                ])
             )
         if self.current_policy in (RECURSE_ALWAYS, RECURSE_NEVER):
             self.permanent_policy = self.current_policy
@@ -1300,7 +1277,7 @@ class ExtractorBuilder(object):
         ("tar", "gzip", "tar.gz", "tgz"),
         ("tar", "lzma", "tar.lzma", "tlz"),
         ("tar", "xz", "tar.xz", "txz"),
-        ("tar", "lz", "tar.lz"),
+        ("tar", "lzip", "tar.lz"),
         ("tar", "compress", "tar.Z", "taz"),
         ("tar", "lrz", "tar.lrz"),
         ("tar", "zstd", "tar.zst"),
@@ -1376,9 +1353,7 @@ class ExtractorBuilder(object):
 
     def try_by_magic(self, filename):
         try:
-            process = subprocess.Popen(
-                ["file", "-zL", filename], stdout=subprocess.PIPE
-            )
+            process = subprocess.Popen(["file", "-zL", filename], stdout=subprocess.PIPE)
             status = process.wait()
             if status != 0:
                 return []
@@ -1473,16 +1448,16 @@ class ExtractionAction(BaseAction):
                 print("%s/" % (filename,))
                 new_filenames = os.listdir(filename)
                 new_filenames = sorted(new_filenames, key=cmp_to_key(reverser))
-                filenames.extend(
-                    [pathjoin(filename, new_filename) for new_filename in new_filenames]
-                )
+                filenames.extend([
+                    pathjoin(filename, new_filename) for new_filename in new_filenames
+                ])
             else:
                 print(filename)
 
     def run(self, filename, extractor):
         self.current_filename = filename
         error = (
-            self.report(extractor.extract, self.options.batch)
+            self.report(extractor.extract, self.options.batch, self.options.password)
             or self.report(self.get_handler, extractor)
             or self.report(self.current_handler.handle)
             or self.report(self.show_extraction, extractor)
@@ -1559,13 +1534,11 @@ class ExtractorApplication(object):
         # get the lists of built-in extensions and combine them
         ext_map_base = set(ExtractorBuilder.extension_map.keys())
         ext_map = set(
-            itertools.chain(
-                *[
-                    x["extensions"]
-                    for x in ExtractorBuilder.extractor_map.values()
-                    if "extensions" in x
-                ]
-            )
+            itertools.chain(*[
+                x["extensions"]
+                for x in ExtractorBuilder.extractor_map.values()
+                if "extensions" in x
+            ])
         )
         ext_map = ext_map_base.union(ext_map)
 
@@ -1619,10 +1592,7 @@ class ExtractorApplication(object):
             "--one-entry",
             dest="one_entry_default",
             default=None,
-            help=(
-                "specify extraction policy for one-entry "
-                + "archives: inside/rename/here"
-            ),
+            help=("specify extraction policy for one-entry " + "archives: inside/rename/here"),
         )
         parser.add_option(
             "-n",
@@ -1632,6 +1602,13 @@ class ExtractorApplication(object):
             default=False,
             help="don't ask how to handle special cases",
         )
+        parser.add_option(
+            "-p",
+            "--password",
+            dest="password",
+            default=None,
+            help="provide a password for password-protected archives",
+        )
         parser.add_option(
             "-o",
             "--overwrite",
@@ -1737,9 +1714,7 @@ class ExtractorApplication(object):
             self.current_extractor = extractor  # For the abort() method.
             error = self.action.run(filename, extractor)
             if error:
-                errors.append(
-                    (extractor.file_type, extractor.encoding, error, extractor.stderr)
-                )
+                errors.append((extractor.file_type, extractor.encoding, error, extractor.stderr))
                 if extractor.target is not None:
                     self.clean_destination(extractor.target)
             else:
